@@ -87,7 +87,7 @@
 
 #define N2L(CHAR_ARRAY, LONG_VAL) \
    { \
-       unsigned char *                  _char_array_ = CHAR_ARRAY; \
+       const unsigned char *                _char_array_ = CHAR_ARRAY; \
        (LONG_VAL)  = ((*(_char_array_++)) << 24) & 0xff000000; \
        (LONG_VAL) |= ((*(_char_array_++)) << 16) & 0xff0000; \
        (LONG_VAL) |= ((*(_char_array_++)) << 8) & 0xff00; \
@@ -108,6 +108,33 @@
        *(_char_array_++) = (unsigned char) ((SHORT) & 0xff); \
    } 
 
+#define U642N(U64VAL, CHAR_ARRAY) \
+    { \
+        unsigned char *             _char_array_ = CHAR_ARRAY; \
+        *(_char_array_++) = (unsigned char) (((U64VAL) >> 56) & 0xff); \
+        *(_char_array_++) = (unsigned char) (((U64VAL) >> 48) & 0xff); \
+        *(_char_array_++) = (unsigned char) (((U64VAL) >> 40) & 0xff); \
+        *(_char_array_++) = (unsigned char) (((U64VAL) >> 32) & 0xff); \
+        *(_char_array_++) = (unsigned char) (((U64VAL) >> 24) & 0xff); \
+        *(_char_array_++) = (unsigned char) (((U64VAL) >> 16) & 0xff); \
+        *(_char_array_++) = (unsigned char) (((U64VAL) >>  8) & 0xff); \
+        *(_char_array_++) = (unsigned char) (((U64VAL)      ) & 0xff); \
+    }
+
+#define N2U64(CHAR_ARRAY, U64VAL) \
+    { \
+        const unsigned char *       _char_array_ = CHAR_ARRAY; \
+        uint64_t                    _u64val_ = 0; \
+        _u64val_ = (((uint64_t)(*(_char_array_++))) << 56) & 0xff00000000000000; \
+        _u64val_ = (((uint64_t)(*(_char_array_++))) << 48) & 0xff000000000000; \
+        _u64val_ = (((uint64_t)(*(_char_array_++))) << 40) & 0xff0000000000; \
+        _u64val_ = (((uint64_t)(*(_char_array_++))) << 32) & 0xff00000000; \
+        _u64val_ = (((uint64_t)(*(_char_array_++))) << 24) & 0xff000000; \
+        _u64val_ = (((uint64_t)(*(_char_array_++))) << 16) & 0xff0000; \
+        _u64val_ = (((uint64_t)(*(_char_array_++))) <<  8) & 0xff00; \
+        _u64val_ = (((uint64_t)(*(_char_array_++)))      ) & 0xff; \
+        (U64VAL) = _u64val_; \
+    }
 /* Compare OIDs */
 
 #define g_OID_equal(o1, o2) \
@@ -148,6 +175,19 @@ typedef struct gss_ctx_id_desc_struct{
     OM_uint32                           req_flags;
     OM_uint32                           ctx_flags;
     int                                 cred_obtained;
+#if OPENSSL_VERSION_NUMBER >= 0x10000100L
+    /** For GCM ciphers, sequence number of next read MAC token */
+    uint64_t                            mac_read_sequence;
+    /** For GCM ciphers, sequence number of next write MAC token */
+    uint64_t                            mac_write_sequence;
+    /** For GCM ciphers, key for MAC token generation/validation */
+    unsigned char *                     mac_key;
+    /**
+     * For GCM ciphers, fixed part of the IV for MAC token
+     * generation/validation
+     */
+    unsigned char *                     mac_iv_fixed;
+#endif
     SSL *                               gss_ssl; 
     BIO *                               gss_rbio;
     BIO *                               gss_wbio;
@@ -170,6 +210,11 @@ gss_OID_desc *                          gss_nt_host_ip;
 extern
 gss_OID_desc *                          gss_nt_x509;
 
+extern
+globus_bool_t                           globus_i_backward_compatible_mic;
+extern
+globus_bool_t                           globus_i_accept_backward_compatible_mic;
+
 #define GLOBUS_GSS_C_NT_HOST_IP         gss_nt_host_ip
 #define GLOBUS_GSS_C_NT_X509            gss_nt_x509
 
@@ -179,4 +224,21 @@ globus_thread_once_t                    once_control;
 void
 globus_l_gsi_gssapi_activate_once(void);
 
+OM_uint32
+globus_i_gss_get_hash(
+    OM_uint32                          *minor_status,
+    const gss_ctx_id_t                  context_handle,
+    const EVP_MD **                     hash,
+    const EVP_CIPHER **                 cipher);
+
+
+OM_uint32
+globus_i_gssapi_gsi_gmac(
+    OM_uint32 *                         minor_status,
+    const EVP_CIPHER *                  evp_cipher,
+    const unsigned char *               iv,
+    const unsigned char *               key,
+    const gss_buffer_desc              *message_buffer,
+    unsigned char                       tag[static 16]);
+
 #endif /* GSSAPI_OPENSSL_H */
